#include "esp_camera.h"
#include "FS.h"
#include "SD_MMC.h"
#include "driver/rtc_io.h"
#include <time.h>
#include <ArduinoJson.h>

// Global variables for configuration
int sleepTime = 10; // Default sleep time in seconds
bool useFlash = false; // Default flash option

void loadConfiguration() {
  File configFile = SD_MMC.open("/config.json", FILE_READ);
  if (!configFile) {
    Serial.println("Failed to open configuration file. Using defaults.");
    return;
  }

  // Allocate a buffer to store the file content
  StaticJsonDocument<256> doc;
  DeserializationError error = deserializeJson(doc, configFile);
  if (error) {
    Serial.println("Failed to parse configuration file. Using defaults.");
    configFile.close();
    return;
  }

  // Load configuration values
  if (doc.containsKey("sleep_time")) {
    sleepTime = doc["sleep_time"];
  }
  if (doc.containsKey("flash")) {
    useFlash = doc["flash"];
  }

  Serial.printf("Configuration loaded: sleep_time=%d, flash=%s\n", sleepTime, useFlash ? "true" : "false");
  configFile.close();
}

// === Camera Configuration ===
void startCamera() {
  camera_config_t config;
  config.ledc_channel = LEDC_CHANNEL_0;
  config.ledc_timer = LEDC_TIMER_0;
  config.pin_d0 = 5;
  config.pin_d1 = 18;
  config.pin_d2 = 19;
  config.pin_d3 = 21;
  config.pin_d4 = 36;
  config.pin_d5 = 39;
  config.pin_d6 = 34;
  config.pin_d7 = 35;
  config.pin_xclk = 0;
  config.pin_pclk = 22;
  config.pin_vsync = 25;
  config.pin_href = 23;
  config.pin_sscb_sda = 26;
  config.pin_sscb_scl = 27;
  config.pin_pwdn = 32;
  config.pin_reset = -1;
  config.xclk_freq_hz = 20000000;
  config.pixel_format = PIXFORMAT_JPEG;

  if(psramFound()) {
    config.frame_size = FRAMESIZE_XGA;
    config.jpeg_quality = 10;
    config.fb_count = 2;
  } else {
    config.frame_size = FRAMESIZE_SVGA;
    config.jpeg_quality = 12;
    config.fb_count = 1;
  }

  esp_err_t err = esp_camera_init(&config);
  if (err != ESP_OK) {
    Serial.printf("Camera init failed: %s\n", esp_err_to_name(err));
    while (true);
  }
    // Access the camera sensor settings
  sensor_t *s = esp_camera_sensor_get();
  if (s) {
    s->set_gain_ctrl(s, 1);       // Auto gain on
    s->set_exposure_ctrl(s, 1);   // Auto exposure on
    s->set_awb_gain(s, 1);        // Auto white balance on
    s->set_brightness(s, 1);      // Set brightness to 1 (-2 to 2)
  } else {
    Serial.println("Failed to access camera sensor settings");
  }
}

// Function to find the next available filename
String getNextFilename() {
  File root = SD_MMC.open("/");
  if (!root) {
    Serial.println("Failed to open root directory");
    return "/img_1.jpg"; // Default to the first image
  }

  Serial.println("Root directory opened successfully");

  int maxIndex = 0;
  File file = root.openNextFile();
  while (file) {
    String name = file.name();
    Serial.println("Found file: " + name); // Debug: Print each file name

    // Ensure the filename matches the pattern "img_*.jpg"
    if (name.startsWith("img_") && name.endsWith(".jpg")) {
      Serial.println("File matches pattern: " + name); // Debug: Matching file
      int start = name.indexOf('_') + 1; // Find the position after '_'
      int end = name.lastIndexOf('.');  // Find the position before '.'
      if (start > 0 && end > start) {   // Ensure valid indices
        String numberStr = name.substring(start, end); // Extract the number part
        int number = numberStr.toInt(); // Convert to integer
        Serial.printf("Extracted number: %d\n", number); // Debug: Extracted number
        if (number > maxIndex) {
          maxIndex = number; // Update maxIndex if a larger number is found
        }
      } else {
        Serial.println("Failed to extract number from filename");
      }
    }
    file = root.openNextFile();
  }
  root.close();

  Serial.printf("Max index found: %d\n", maxIndex); // Debug: Max index
  return "/img_" + String(maxIndex + 1) + ".jpg"; // Return the next filename
}

void setup() {
  Serial.begin(115200);
  delay(1000);
  Serial.println("Booting...");

  // Initialize the flash LED pin
  pinMode(4, OUTPUT);
  digitalWrite(4, LOW); // Turn off the flash LED initially

  startCamera();

  if (!SD_MMC.begin("/sdcard", true)) {
    Serial.println("SD Card mount failed");
    while (true);
  }

  Serial.println("SD Card mounted");

  // Load configuration from the SD card
  loadConfiguration();

  // Turn on the flash LED if enabled in the configuration
  if (useFlash) {
    digitalWrite(4, HIGH);
    delay(100); // Allow the LED to stabilize
  }

  camera_fb_t * fb = esp_camera_fb_get();
  if (!fb) {
    Serial.println("Camera capture failed");
    if (useFlash) {
      digitalWrite(4, LOW); // Turn off the flash LED
    }
    return;
  }

  // Get the next available filename
  String filename = getNextFilename();
  File file = SD_MMC.open(filename.c_str(), FILE_WRITE);
  if (!file) {
    Serial.println("Failed to write file");
  } else {
    file.write(fb->buf, fb->len);
    file.close();
    Serial.println("Image saved: " + filename);
  }

  esp_camera_fb_return(fb);

  // Turn off the flash LED after capturing the image
  if (useFlash) {
    digitalWrite(4, LOW);
  }
  pinMode(4, INPUT_PULLDOWN); // Set the pin to input with a pull-down resistor

  Serial.printf("Going to sleep for %d seconds...\n", sleepTime);

  esp_sleep_enable_timer_wakeup((uint64_t)sleepTime * 1000000ULL); // Sleep time in microseconds
  esp_deep_sleep_start();
}

void loop() {
  // Not used due to deep sleep
}
